@@ -97,7 +97,7 @@ if (VERBOSE_LEVEL>7) fprintf(stdout,"%s:%i sopen_cfs_read started [%s]\n",__FILE
 			hdr->EVENT.POS = (typeof(hdr->EVENT.POS)) realloc(hdr->EVENT.POS, (hdr->EVENT.N + NumberOfDataSections - 1) * sizeof(*hdr->EVENT.POS));
 		}
 
-if (VERBOSE_LEVEL>7) fprintf(stdout,"CFS 131 - %d,%d,%d,0x%x,0x%x,0x%x,%d,0x%x\n",hdr->NS,n,d,FileHeaderSize,DataHeaderSize,LastDataSectionHeaderOffset,NumberOfDataSections,leu32p(hdr->AS.Header+0x86));
+if (VERBOSE_LEVEL>7) fprintf(stdout,"%s (line %i) CFS - %d,%d,%d,0x%x,0x%x,0x%x,%d,0x%x\n",__FILE__,__LINE__,hdr->NS,n,d,FileHeaderSize,DataHeaderSize,LastDataSectionHeaderOffset,NumberOfDataSections,leu32p(hdr->AS.Header+0x86));
 
 		/* channel information */
 		hdr->CHANNEL = (CHANNEL_TYPE*)realloc(hdr->CHANNEL, hdr->NS * sizeof(CHANNEL_TYPE));
@@ -123,6 +123,9 @@ if (VERBOSE_LEVEL>7) fprintf(stdout,"CFS 131 - %d,%d,%d,0x%x,0x%x,0x%x,%d,0x%x\n
 			while (isspace(hc->Label[len])) len--;		// remove trailing blanks
 			hc->Label[len+1] = 0;
 
+if (VERBOSE_LEVEL>7) fprintf(stdout,"%s(line %i) Channel #%i: %i<%s>/%i<%s>\n",__FILE__,__LINE__,k+1, H2[22 + k*H2LEN], H2 + 23 + k*H2LEN, H2[32 + k*H2LEN], H2 + 33 + k*H2LEN);
+
+
 			hc->PhysDimCode  = PhysDimCode (H2 + 22 + 1 + k*H2LEN);			// Y axis units
 			xPhysDimScale[k] = PhysDimScale(PhysDimCode(H2 + 32 + 1 + k*H2LEN));	// X axis units
 
@@ -143,7 +146,7 @@ if (VERBOSE_LEVEL>7) fprintf(stdout,"CFS 131 - %d,%d,%d,0x%x,0x%x,0x%x,%d,0x%x\n
 			hc->LowPass  = NAN;
 			hc->HighPass = NAN;
 			hc->Notch    = NAN;
-if (VERBOSE_LEVEL>7) fprintf(stdout,"Channel #%i: [%s](%i/%i) <%s>/<%s> ByteSpace%i,Next#%i\n",k+1, H2 + 1 + k*H2LEN, dataType, H2[43], H2 + 23 + k*H2LEN, H2 + 33 + k*H2LEN, leu16p(H2+44+k*H2LEN), leu16p(H2+46+k*H2LEN));
+if (VERBOSE_LEVEL>7) fprintf(stdout,"%s(line %i) Channel #%i: [%s](%i/%i) <%s>/<%s> ByteSpace%i,Next#%i\n",__FILE__,__LINE__,k+1, H2 + 1 + k*H2LEN, dataType, H2[43], H2 + 23 + k*H2LEN, H2 + 33 + k*H2LEN, leu16p(H2+44+k*H2LEN), leu16p(H2+46+k*H2LEN));
 		}
 		hdr->AS.bpb = bpb;
 
@@ -306,6 +309,8 @@ if (VERBOSE_LEVEL>7) fprintf(stdout,"\n[DS#%3i] 0x%x 0x%x [0x%x 0x%x szChanData=
 			hdr->AS.first  = 0;
 			hdr->AS.length = 0;
 			char flag_firstchan = 1; 
+			uint32_t xspr0= 0;
+
 			for (k = 0; k < hdr->NS; k++) {
 				uint8_t *pos = hdr->AS.Header + datapos + 30 + 24 * k;
 
@@ -315,26 +320,31 @@ if (VERBOSE_LEVEL>7) fprintf(stdout,"\n[DS#%3i] 0x%x 0x%x [0x%x 0x%x szChanData=
 				uint32_t xspr = leu32p(pos+4);
 				float Cal    = lef32p(pos+8);
 				float Off    = lef32p(pos+12);
-				double Xcal  = lef32p(pos+16);
-				double Xoff  = lef32p(pos+20);// unused
+				double XCal  = lef32p(pos+16);
+				double XOff  = lef32p(pos+20);// unused
+				if (VERBOSE_LEVEL>7) fprintf(stdout,"%s (line %i): %i/%i %i/%i %f+%f change SPR:%i->%i, Cal:%f->%f, Off: %f->%f\n",__FILE__,__LINE__, (int)m, (int)NumberOfDataSections, (int)k,(int)hdr->NS, XCal, XOff, (int)hc->SPR,(int)xspr,hc->Cal,Cal,hc->Off,Off);
+
+				if (k==0) xspr0 = xspr;
+				else if (xspr0 != xspr) {
+					if (VERBOSE_LEVEL>7) fprintf(stdout,"Error %s (line %i): %i/%i %i/%i change SPR:%i->%i\n",__FILE__,__LINE__, (int)m, (int)NumberOfDataSections, (int)k,(int)hdr->NS,(int)hc->SPR,(int)xspr);
+					biosigERROR(hdr, B4C_FORMAT_UNSUPPORTED, "CED/CFS: samples per record change - this is not supported yet.\n");
+				}
+
 				if (m > 0) {
-					if ( (hc->SPR != xspr)
-					  || (hc->Cal != Cal)
+					if ( (hc->Cal != Cal)
 					  || (hc->Off != Off)
 					   )
 					biosigERROR(hdr, B4C_FORMAT_UNSUPPORTED, "CED/CFS: channel properties changes between segments - this is not supported yet.\n");
 				}
 				else {
-					hc->SPR = xspr;
 					hc->Cal = Cal;
 					hc->Off = Off;
 				}
 
+if (VERBOSE_LEVEL>7) fprintf(stdout,"CFS 408: %i #%i: SPR=%i=%i=%i  x%f+-%f %i x%g %g %g\n",m,k,spr,(int)SPR,hc->SPR,hc->Cal,hc->Off,hc->bi,xPhysDimScale[k], XCal, XOff);
 
-if (VERBOSE_LEVEL>7) fprintf(stdout,"CFS 408: %i #%i: SPR=%i=%i=%i  x%f+-%f %i x%g %g %g\n",m,k,spr,(int)SPR,hc->SPR,hc->Cal,hc->Off,hc->bi,xPhysDimScale[k], Xcal, Xoff);
-
-				double Fs = 1.0 / (xPhysDimScale[k] * Xcal);
-				if ( (hc->OnOff == 0) || (Xcal == 0.0) ) {
+				double Fs = 1.0 / (xPhysDimScale[k] * XCal);
+				if ( (hc->OnOff == 0) || (XCal == 0.0) ) {
 					; // do nothing:  
 				}
 				else if (flag_firstchan) {
@@ -345,8 +355,6 @@ if (VERBOSE_LEVEL>7) fprintf(stdout,"CFS 408: %i #%i: SPR=%i=%i=%i  x%f+-%f %i x
 					biosigERROR(hdr, B4C_FORMAT_UNSUPPORTED, "CED/CFS: different sampling rates are not supported\n");
 				}
 
-				if (hc->OnOff && (spr < hc->SPR)) 
-					spr = hc->SPR;
 				sz  += hc->SPR * GDFTYP_BITS[hc->GDFTYP] >> 3;
 				assert(hc->bi == bpb);
 				bpb += GDFTYP_BITS[hc->GDFTYP]>>3;	// per single sample
@@ -354,7 +362,7 @@ if (VERBOSE_LEVEL>7) fprintf(stdout,"CFS 408: %i #%i: SPR=%i=%i=%i  x%f+-%f %i x
 				
 			}
 
-			SPR += spr;
+			SPR += xspr0;
 			SZ  += sz;
 			if (hdr->AS.bpb != bpb)
 				biosigERROR(hdr, B4C_FORMAT_UNSUPPORTED, "CED/CFS: channel properties changes between segments - this is not supported yet.\n");
@@ -384,24 +392,29 @@ if (VERBOSE_LEVEL>7) fprintf(stdout,"CFS 408: %i #%i: SPR=%i=%i=%i  x%f+-%f %i x
 			hdr->AS.first  = 0;
 			hdr->AS.length = 0;
 			char flag_firstchan = 1;
+			uint32_t xspr0= 0;
 			for (k = 0; k < hdr->NS; k++) {
 				uint8_t *pos = hdr->AS.Header + datapos + 30 + 24 * k;
 
 				CHANNEL_TYPE *hc = hdr->CHANNEL + k;
 
 				uint32_t bi = leu32p(pos);
+				uint32_t xspr = leu32p(pos+4);
 				hc->SPR     = leu32p(pos+4);
 /*
 				hc->Cal     = lef32p(pos+8);
 				hc->Off     = lef32p(pos+12);
 */
-				double Xcal = lef32p(pos+16);
-				double Xoff = lef32p(pos+20);// unused
+				double XCal = lef32p(pos+16);
+				double XOff = lef32p(pos+20);// unused
 
-if (VERBOSE_LEVEL>7) fprintf(stdout,"CFS 409: %i #%i: SPR=%i=%i=%i  x%f+-%f %i x%g %g %g\n",m,k,spr,(int)SPR,hc->SPR,hc->Cal,hc->Off,hc->bi,xPhysDimScale[k], Xcal, Xoff);
+				if (k==0) xspr0 = xspr;
+				assert(xspr0 == xspr);
 
-				double Fs = 1.0 / (xPhysDimScale[k] * Xcal);
-				if ( (hc->OnOff == 0) || (Xcal == 0.0) ) {
+if (VERBOSE_LEVEL>7) fprintf(stdout,"CFS 409: %i #%i: SPR=%i=%i=%i  x%f+-%f %i x%g %g %g %g %g\n",m,k,spr, (int)SPR, hc->SPR, hc->Cal, hc->Off, hc->bi, xPhysDimScale[k], lef32p(pos+8), lef32p(pos+12), XCal, XOff);
+
+				double Fs = 1.0 / (xPhysDimScale[k] * XCal);
+				if ( (hc->OnOff == 0) || (XCal == 0.0) ) {
 					; // do nothing:
 				}
 				else if (flag_firstchan) {
@@ -412,12 +425,10 @@ if (VERBOSE_LEVEL>7) fprintf(stdout,"CFS 409: %i #%i: SPR=%i=%i=%i  x%f+-%f %i x
 					biosigERROR(hdr, B4C_FORMAT_UNSUPPORTED, "CED/CFS: different sampling rates are not supported\n");
 				}
 
-				if (hc->OnOff && (spr < hc->SPR))
-					spr = hc->SPR;
-				sz  += hc->SPR * GDFTYP_BITS[hc->GDFTYP] >> 3;
+				sz  += xspr * GDFTYP_BITS[hc->GDFTYP] >> 3;
 				hc->bi = bpb;
 				bpb += GDFTYP_BITS[hc->GDFTYP]>>3;	// per single sample
-				hdr->AS.length += hc->SPR;
+				hdr->AS.length += xspr;
 
 			}
 
@@ -450,7 +461,9 @@ if (VERBOSE_LEVEL>7) fprintf(stdout,"CFS 412 #%i %i %i %i %i: %i @%p %i\n", k, h
 					}
 
 					size_t k2;
-					for (k2 = 0; k2 < hc->SPR; k2++) {
+
+					for (k2 = 0; k2 < xspr0; k2++) {
+
 						uint8_t *ptr = srcaddr + memoffset + k2*stride;
 
 if (VERBOSE_LEVEL>8) fprintf(stdout,"512 %i %i %i %i %i \n",(int)stride,(int)hc->bi,(int)SPR,(int)k2,(int)hdr->AS.bpb);
@@ -486,12 +499,8 @@ if (VERBOSE_LEVEL>8) fprintf(stdout,"512 %i %i %i %i %i \n",(int)stride,(int)hc-
 							biosigERROR(hdr, B4C_FORMAT_UNSUPPORTED, "CED/CFS: invalid data type");
 						}
 
-					   	if (hc->OnOff) {
-							/* TODO: channels with less samples are currently ignored - resampling or ignoring the channel ? */
-//							*(double*) (hdr->AS.rawdata + (k + (SPR + k2)*hdr->NS) * sizeof(double)) = val * hc->Cal + hc->Off;
-							continue;
-					   	}
-						
+						if (hc->OnOff) continue;
+
 						if (!strncmp(hc->Label,"Marker",6) && hc->PhysDimCode==2176 && hc->GDFTYP==5 && next != 0) { 
 							// matrix data might contain time markers. 						
 
@@ -517,14 +526,14 @@ if (VERBOSE_LEVEL>8) fprintf(stdout,"512 %i %i %i %i %i \n",(int)stride,(int)hc-
 				}
 			}
 
-			SPR += spr;
+			SPR += xspr0;
 			SZ  += sz;
 
 			datapos = leu32p(hdr->AS.Header + datapos);
 		}
 		free(DATAPOS);
 
-if (VERBOSE_LEVEL>7) fprintf(stdout,"CFS 419: SPR=%i=%i NRec=%i  @%p\n",(int)SPR,hdr->SPR,(int)hdr->NRec, hdr->AS.rawdata);
+if (VERBOSE_LEVEL>7) fprintf(stdout,"%s (line %i): SPR=%i=%i NRec=%i  @%p\n",__FILE__,__LINE__,(int)SPR,hdr->SPR,(int)hdr->NRec, hdr->AS.rawdata);
 
 		// set variables such that sread_raw does not attempt to reload the data
 		hdr->AS.first = 0;
@@ -533,7 +542,6 @@ if (VERBOSE_LEVEL>7) fprintf(stdout,"CFS 419: SPR=%i=%i NRec=%i  @%p\n",(int)SPR
 			hdr->SPR = 0; 	
 		}
 		else  {
-			hdr->FLAG.UCAL = 1;
 			hdr->SPR       = 1;
 			hdr->NRec      = SPR;
 			hdr->AS.length = SPR;
@@ -556,34 +564,36 @@ if (VERBOSE_LEVEL>7) fprintf(stdout,"CFS 429: SPR=%i=%i NRec=%i\n",(int)SPR,hdr-
 		}
 
 		for (k = 0; k < hdr->NS; k++) {
-			switch (hdr->CHANNEL[k].GDFTYP) {
+			CHANNEL_TYPE *hc = hdr->CHANNEL + k;
+			switch (hc->GDFTYP) {
 			case 0:
 			case 1:
-				hdr->CHANNEL[k].DigMax  =  127;
-				hdr->CHANNEL[k].DigMin  = -128;
+				hc->DigMax  =  127;
+				hc->DigMin  = -128;
 				break;
 			case 2:
-				hdr->CHANNEL[k].DigMax  =  255;
-				hdr->CHANNEL[k].DigMin  =  0;
+				hc->DigMax  =  255;
+				hc->DigMin  =  0;
 				break;
 			case 3:
-				hdr->CHANNEL[k].DigMax  = (int16_t)0x7fff;
-				hdr->CHANNEL[k].DigMin  = (int16_t)0x8000;
+				hc->DigMax  = (int16_t)0x7fff;
+				hc->DigMin  = (int16_t)0x8000;
 				break;
 			case 4:
-				hdr->CHANNEL[k].DigMax  = 0xffff;
-				hdr->CHANNEL[k].DigMin  = 0;
+				hc->DigMax  = 0xffff;
+				hc->DigMin  = 0;
 				break;
 			case 16:
 			case 17:
-				hdr->CHANNEL[k].DigMax  =  1e9;
-				hdr->CHANNEL[k].DigMin  = -1e9;
+				hc->DigMax  =  1e9;
+				hc->DigMin  = -1e9;
 				break;
 			}
-			hdr->CHANNEL[k].PhysMax = hdr->CHANNEL[k].DigMax * hdr->CHANNEL[k].Cal + hdr->CHANNEL[k].Off;
-			hdr->CHANNEL[k].PhysMin = hdr->CHANNEL[k].DigMin * hdr->CHANNEL[k].Cal + hdr->CHANNEL[k].Off;
+if (VERBOSE_LEVEL>7) fprintf(stdout,"%s (%i): %f %f %f %f %f %f\n",__FILE__,__LINE__,hc->PhysMax, hc->DigMax, hc->Cal, hc->Off,hc->PhysMin, hc->DigMin );
+			hc->PhysMax = hc->DigMax * hc->Cal + hc->Off;
+			hc->PhysMin = hc->DigMin * hc->Cal + hc->Off;
+if (VERBOSE_LEVEL>7) fprintf(stdout,"%s (%i): %f %f %f %f %f %f\n",__FILE__,__LINE__,hc->PhysMax, hc->DigMax, hc->Cal, hc->Off,hc->PhysMin, hc->DigMin );
 		}
-		hdr->FLAG.UCAL = 1;
 
 #undef H1LEN
 }
