@@ -319,7 +319,35 @@ void deallocEN1064(en1064_t en1064) {
 }
 
 
-int decode_scp_text(HDRTYPE *hdr, size_t inbytesleft, char *input, size_t outbytesleft, char *output) {
+/*
+  decode_scp_text converts SCP text strings in various language encodings into UTF-8.
+  hdr is used to identify the language support code of EN1064+A1:2007
+  versionSection is used to handle older versions (specifically 10) in a reasonable way.
+
+  input can be a string that without a 0-terminated character;
+  the end of the input string is determined by inbytesleft.
+  output must be of size outbytesleft+1.
+*/
+int decode_scp_text(HDRTYPE *hdr, size_t inbytesleft, char *input, size_t outbytesleft, char *output, uint8_t versionSection) {
+
+#ifdef DEBUG
+	const char *start = output;
+#endif
+
+	int exitcode = 0;
+	switch (versionSection) {
+	case 13:
+	case 20:
+		break;  // use language conversion code below
+	case 10:
+		exitcode =  0;
+	default:
+		exitcode = -1;	// unknown version - do not know whether this is the correct way of doing it.
+		outbytesleft = min(inbytesleft,outbytesleft);
+		memcpy(output,input,outbytesleft);
+		output[outbytesleft]=0;
+		return(exitcode);
+	}
 
 #if  defined(_ICONV_H) || defined (_LIBICONV_H)
 /*
@@ -379,28 +407,64 @@ int decode_scp_text(HDRTYPE *hdr, size_t inbytesleft, char *input, size_t outbyt
 		return -1;
 	}
 
-	if (VERBOSE_LEVEL>7) fprintf(stdout,"%s(%i) decode_scp_text: input=<%s>\n", __FILE__, __LINE__, input);
+	errno = 0; // reset error status
+	int errsv;
+	if (input[inbytesleft-1]==0) {
 
-	iconv(cd, &input, &inbytesleft, &output, &outbytesleft);
-	int errsv = errno;
+		if (VERBOSE_LEVEL>7) fprintf(stdout,"%s(%i) decode_scp_text: input=<%s>%i,%i\n", __FILE__, __LINE__, input,(int)inbytesleft,(int)outbytesleft);
 
+		// input string is 0-terminated
+		iconv(cd, &input, &inbytesleft, &output, &outbytesleft);
+		errsv = errno;
+	}
+	else if (inbytesleft < 64) {
+		/* In case the string is not 0-terminated,
+		 * the string is copied to make it 0-terminated
+		 */
+		char buf[64];
+		char *tmpstr=buf;
+		memcpy(buf,input,inbytesleft);
+		tmpstr[inbytesleft++]=0;
+
+		if (VERBOSE_LEVEL>7) fprintf(stdout,"%s(%i) decode_scp_text: input=<%s>%i,%i\n", __FILE__, __LINE__, input,(int)inbytesleft,(int)outbytesleft);
+
+		iconv(cd, &tmpstr, &inbytesleft, &output, &outbytesleft);
+		errsv = errno;
+	}
+	else {
+		/* In case the string is not 0-terminated,
+		 * the string is copied to make it 0-terminated
+		 */
+		char *tmpstr=malloc(inbytesleft+1);
+		char *bakstr=tmpstr;
+		strncpy(tmpstr,(char*)input,inbytesleft);
+		tmpstr[inbytesleft]=0;
+		inbytesleft++;
+
+		if (VERBOSE_LEVEL>7) fprintf(stdout,"%s(%i) decode_scp_text: input=<%s>%i,%i\n", __FILE__, __LINE__, tmpstr,(int)inbytesleft,(int)outbytesleft);
+
+		iconv(cd, &tmpstr, &inbytesleft, &output, &outbytesleft);
+		errsv = errno;
+		free(bakstr);
+	}
 	if (errsv)
 		biosigERROR(hdr, B4C_CHAR_ENCODING_UNSUPPORTED, "conversion of SCP text failed");
 
-
-	if (VERBOSE_LEVEL>7) fprintf(stdout,"%s(%i) decode_scp_text: output=<%s>\n", __FILE__, __LINE__, output);
+#ifdef DEBUG
+	if (VERBOSE_LEVEL>7) fprintf(stdout,"%s(%i) decode_scp_text: [e%i ] output=<%s>%i,%i\n", __FILE__, __LINE__, errsv,start,inbytesleft,outbytesleft);
+#endif
 
 	return (iconv_close(cd) || errsv);
 
 #else  // if neither _ICONV_H nor _LIBCONV_H are defined
 
-	if ((LanguageSupportCode & 0x01) != 0) {
+	if ((LanguageSupportCode & 0xFE) != 0) {
 		biosigERROR(hdr, B4C_CHAR_ENCODING_UNSUPPORTED, "SCP character encoding not supported");
 	}
 	else	// ASCII encoding is UTF-8 compatible - no convesion needed
 		strncpy(output, input, min(inbytesleft, outbytesleft+1));
 
-	return(LanguageSupportCode & 0x01);
+	return(LanguageSupportCode & 0xFE);
 
 #endif
 
@@ -569,9 +633,11 @@ EXTERN_C int sopen_SCP_read(HDRTYPE* hdr) {
 
 		PtrCurSect = ptr+sectionStart;
 		crc 	   = leu16p(PtrCurSect);
+		/*
 		uint16_t tmpcrc = CRCEvaluate((uint8_t*)(PtrCurSect+2),len-2);
 		uint8_t versionSection  = *(ptr+sectionStart+8);
 		uint8_t versionProtocol = *(ptr+sectionStart+9);
+		*/
 		// future versions might not need to do this, because language encoding is fixed (i.e. known).
 
 			uint32_t len1;
@@ -592,19 +658,16 @@ EXTERN_C int sopen_SCP_read(HDRTYPE* hdr) {
 
 	for (K=1; K<NSections; K++)	{
 
-	if (VERBOSE_LEVEL>7)
-		fprintf(stdout,"%s(%i) ",hdr->FileName,K);
-		
 		curSect           = section[K].ID;
 		len		  = section[K].length;
 		sectionStart 	  = section[K].index;
 
 	if (VERBOSE_LEVEL>7)
-		fprintf(stdout,"SCP Section %i %i len=%i secStart=%i HeaderLength=%i\n",K,curSect,len,(int)sectionStart,hdr->HeadLen);
+		fprintf(stdout,"%s (line %i): SCP Section %i %i len=%i secStart=%i HeaderLength=%i\n",__FILE__,__LINE__,K,curSect,len,(int)sectionStart,hdr->HeadLen);
 
 	if (len==0) continue;	 /***** empty section *****/
 		if (sectionStart + len > hdr->HeadLen) {
-			biosigERROR(hdr, B4C_INCOMPLETE_FILE, "SOPEN(SCP-READ): File inclomplete - Section length + start of section is more then total length of header");
+			biosigERROR(hdr, B4C_INCOMPLETE_FILE, "%s (line %i): SOPEN(SCP-READ): File incomplete - Section length + start of section is more then total length of header");
 			break;
 		}
 
@@ -620,11 +683,14 @@ EXTERN_C int sopen_SCP_read(HDRTYPE* hdr) {
 			fprintf(stderr,"Warning SOPEN(SCP-READ): Current Section No does not match field in sections (%i %i)\n",curSect,leu16p(PtrCurSect+2)); 
 		if (len != leu32p(PtrCurSect+4))
 			fprintf(stderr,"Warning SOPEN(SCP-READ): length field in pointer section (%i) does not match length field in sections (%i %i)\n",K,len,leu32p(PtrCurSect+4)); 
-		if (versionSection != 13 && versionSection != 20)
+		if ((versionSection != 13) && (versionSection != 20) && (versionSection != (uint8_t)(hdr->Version*10)))
 			fprintf(stderr,"Warning SOPEN(SCP-READ): Version of section %i is not 13 or 20 but %i. This is not tested.\n", curSect, versionSection);
-		if (versionProtocol != 13 && versionProtocol != 20)
+		if ((versionProtocol != 13) && (versionProtocol != 20) && (versionProtocol != (uint8_t)(hdr->Version*10)))
 			fprintf(stderr,"Warning SOPEN(SCP-READ): Version of Protocol is not 13 or 20 but %i. This is not tested.\n", versionProtocol);
 #endif
+		if (VERBOSE_LEVEL>7)
+			fprintf(stdout,"%s (line %i): SCP Section %i %i len=%i secStart=%i version=%i %i \n",__FILE__,__LINE__, K, curSect, len, (int)sectionStart,(int)versionSection, (int)versionProtocol);
+
 		curSectPos = 16;
 
 		/**** SECTION 0: POINTERS TO DATA AREAS IN THE RECORD ****/
@@ -665,7 +731,7 @@ EXTERN_C int sopen_SCP_read(HDRTYPE* hdr) {
 					if (!hdr->FLAG.ANONYMOUS) {
 						// strncpy(hdr->Patient.Name, (char*)(PtrCurSect+curSectPos),min(len1,MAX_LENGTH_NAME));
 						 // convert to UTF8
-						decode_scp_text(hdr, len1, PtrCurSect+curSectPos, MAX_LENGTH_NAME, hdr->Patient.Name);
+						decode_scp_text(hdr, len1, (char*)PtrCurSect+curSectPos, MAX_LENGTH_NAME, hdr->Patient.Name, versionSection);
 					}
 				}
 				else if (tag==1) {
@@ -675,20 +741,20 @@ EXTERN_C int sopen_SCP_read(HDRTYPE* hdr) {
 							strcat(hdr->Patient.Name,", ");
 							len+=2;
 							// convert to UTF8
-							decode_scp_text(hdr, len1, PtrCurSect+curSectPos, MAX_LENGTH_NAME-len+1, hdr->Patient.Name+len);
+							decode_scp_text(hdr, len1, (char*)PtrCurSect+curSectPos, MAX_LENGTH_NAME-len+1, hdr->Patient.Name+len, versionSection);
 						}
 					}
 				}
 				else if (tag==2) {
-					if (len1 > MAX_LENGTH_PID) {
 #ifndef ANDROID
+					if (len1 > MAX_LENGTH_PID) {
 						fprintf(stdout,"Warning SCP(read): length of Patient Id (section1 tag2) exceeds %i>%i\n",len1,MAX_LENGTH_PID); 
-#endif
 					}
+#endif
+
 					// convert to UTF8
-					decode_scp_text(hdr, len1, PtrCurSect+curSectPos, MAX_LENGTH_PID, hdr->Patient.Id);
+					decode_scp_text(hdr, len1, (char*)PtrCurSect+curSectPos, MAX_LENGTH_PID, hdr->Patient.Id, versionSection);
 					hdr->Patient.Id[MAX_LENGTH_PID] = 0;
-					
 					if (!strcmp(hdr->Patient.Id,"UNKNOWN"))
 						hdr->Patient.Id[0] = 0;
 				}
@@ -699,7 +765,7 @@ EXTERN_C int sopen_SCP_read(HDRTYPE* hdr) {
 							strcat(hdr->Patient.Name," ");
 							len+=1;
 							// convert to UTF8
-							decode_scp_text(hdr, len1, PtrCurSect+curSectPos, MAX_LENGTH_NAME-len+1, hdr->Patient.Name+len);
+							decode_scp_text(hdr, len1, (char*)PtrCurSect+curSectPos, MAX_LENGTH_NAME-len+1, hdr->Patient.Name+len, versionSection);
 						}
 					}
 				}
@@ -802,7 +868,7 @@ EXTERN_C int sopen_SCP_read(HDRTYPE* hdr) {
 					aECG->Section1.Tag14.ACQ_DEV_MANUF  = (char*)(PtrCurSect+curSectPos+36+tmp+1);	// tag 14, byte 38 (ACQ_DEV_MANUF has to be "Manufacturer")
 					
 
-					if (aECG->Section1.Tag14.LANG_SUPP_CODE & 0x01) {
+					if (aECG->Section1.Tag14.LANG_SUPP_CODE & 0xFE) {
 #if _ICONV_H
 						fprintf(stdout, "Warning SCP-ECG: decoding of text strings not ready yet");
 #else
@@ -824,7 +890,7 @@ EXTERN_C int sopen_SCP_read(HDRTYPE* hdr) {
 					hdr->ID.Hospital = malloc(outlen);
 					if (hdr->ID.Hospital) {
 						// convert to UTF8
-						decode_scp_text(hdr, len1, PtrCurSect+curSectPos, outlen, hdr->ID.Hospital);
+						decode_scp_text(hdr, len1, (char*)PtrCurSect+curSectPos, outlen, hdr->ID.Hospital, versionSection);
 						hdr->ID.Hospital[outlen] = 0;
 					}
 				}
@@ -853,7 +919,7 @@ EXTERN_C int sopen_SCP_read(HDRTYPE* hdr) {
 					hdr->ID.Technician = malloc(outlen);
 					if (hdr->ID.Technician) {
 						// convert to UTF8
-						decode_scp_text(hdr, len1, PtrCurSect+curSectPos, outlen, hdr->ID.Technician);
+						decode_scp_text(hdr, len1, (char*)PtrCurSect+curSectPos, outlen, hdr->ID.Technician, versionSection);
 						hdr->ID.Technician[outlen] = 0;
 					}
 				}
